/*
 *
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>

#include "utils/dlist.h"

#include "tdm_common.h"
#include "tdm_object.h"
#include "tdm_log.h"

#include "tdm_module.h"
#include "tdm_device.h"
#include "tdm_port.h"

extern struct dlist_head tdm_module_list;

/*
 * dump device tree to stdout
 */
tdm_status_t tdm_debug_print_devtree(void)
{
	struct dlist_head *m_entry, *d_entry, *p_entry;
	char prefix[20] = { 0 };

#define BRANCH_DELIM(p)	(dlist_is_last(p) ? "\\_" : "+-")
#define LINE_DELIM(p)	(dlist_is_last(p) ? " "  : "|")

	dlist_for_each(&tdm_module_list, m_entry) {
		struct tdm_module *mod = dlist_entry(m_entry, struct tdm_module, module_list);

		if (!tdm_object_is_module(mod)) {
			tdm_error("%p: object in module list is not a module", mod);
			break;
		}

		if (!mod->nr_devices)	/* no devices registered, skip */
			continue;

		printf("%s module \"%s\"\n", BRANCH_DELIM(m_entry), tdm_object_name(mod));
		printf("%s\tDevices:    %d\n", LINE_DELIM(m_entry), mod->nr_devices);
		printf("%s\n", LINE_DELIM(m_entry));

		/* walk module's device list */
		dlist_for_each(&mod->devices, d_entry) {
			struct tdm_device *dev = dlist_entry(d_entry, struct tdm_device, device_list);

			if (!tdm_object_is_device(dev)) {
				tdm_error("%p: object in device list is not a device", dev);
				break;
			}

			sprintf(prefix, "%s", LINE_DELIM(m_entry));
			printf("%s\t%s device \"%s\"\n", prefix, BRANCH_DELIM(d_entry), tdm_object_name(dev));

			sprintf(prefix, "%s\t%s", LINE_DELIM(m_entry), LINE_DELIM(d_entry));
			printf("%s\tPorts:    %d\n", prefix, dev->nr_ports);

			printf("%s\n", prefix);

			/* walk device's port list */
			dlist_for_each(&dev->port_list, p_entry) {
				struct tdm_port *port = dlist_entry(p_entry, struct tdm_port, port_list);

				if (!tdm_object_is_port(port)) {
					tdm_error("%p: object in port list is not a port", port);
					break;
				}

				sprintf(prefix, "%s\t%s", LINE_DELIM(m_entry), LINE_DELIM(d_entry));
				printf("%s\t%s port \"%s\"\n", prefix, BRANCH_DELIM(p_entry), tdm_object_name(port));

				sprintf(prefix, "%s\t%s\t%s", LINE_DELIM(m_entry), LINE_DELIM(d_entry), LINE_DELIM(p_entry));
				printf("%s\tChannels:    %d\n", prefix, port->nr_channels);

				if (!dlist_is_last(p_entry))
					printf("%s\n", prefix);
			}
		}
	}
#undef BRANCH_DELIM
#undef LINE_DELIM

	return TDM_STATUS_SUCCESS;
}
