#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

//#include "tdm_private.h"
#include "tdm_common.h"
#include "tdm_log.h"

/*
 * internal, tdm_log.c
 */
#define TDM_COLOR_NONE	 "\x1b[0m"
#define TDM_COLOR_RED    "\x1b[31m"
#define TDM_COLOR_GREEN  "\x1b[32m"
#define TDM_COLOR_YELLOW "\x1b[36m"
#define TDM_COLOR_BLUE   "\x1b[34m"
#define TDM_COLOR_GRAY   "\x1b[2;37m"
#define TDM_COLOR_WHITE  "\x1b[37m"

#define TDM_COLOR_HALF   "\x1b[2m"
#define TDM_COLOR_BOLD   "\x1b[1m"
#define TDM_COLOR_BLINK  "\x1b[5m"

#define TDM_LOGF_NONE	0x0
#define TDM_LOGF_COLORS	0x1

const static struct tdm_loglevel_name {
	const tdm_loglevel_t id;
	const char *name;
} tdm_loglevel_names[] = {
	{ TDM_LOG_NONE,      "none"      },
	{ TDM_LOG_EMERGENCY, "emergency" },
	{ TDM_LOG_CRITICAL,  "critical"  },
	{ TDM_LOG_ALERT,     "alert"     },
	{ TDM_LOG_ERROR,     "error"     },
	{ TDM_LOG_WARNING,   "warning"   },
	{ TDM_LOG_NOTICE,    "notice"    },
	{ TDM_LOG_INFO,      "info"      },
	{ TDM_LOG_DEBUG,     "debug"     },
};
const static int tdm_loglevel_name_count = ARRAY_SIZE(tdm_loglevel_names);


const static char* tdm_loglevel_colors[] = {
	[TDM_LOG_NONE]      = TDM_COLOR_NONE,
	[TDM_LOG_EMERGENCY] = TDM_COLOR_BLINK TDM_COLOR_RED,
	[TDM_LOG_CRITICAL]  = TDM_COLOR_BLINK TDM_COLOR_RED,
	[TDM_LOG_ALERT]     = TDM_COLOR_BLINK TDM_COLOR_RED,
	[TDM_LOG_ERROR]     = TDM_COLOR_RED,
	[TDM_LOG_WARNING]   = TDM_COLOR_YELLOW,
	[TDM_LOG_NOTICE]    = TDM_COLOR_BOLD TDM_COLOR_BLUE,
	[TDM_LOG_INFO]      = TDM_COLOR_BOLD TDM_COLOR_WHITE,
	[TDM_LOG_DEBUG]     = TDM_COLOR_GRAY
};
const static int tdm_loglevel_color_count = ARRAY_SIZE(tdm_loglevel_colors);

/* public api */
TDM_API const char *
tdm_loglevel_get_name(const tdm_loglevel_t level)
{
	for (int i = 0; i < tdm_loglevel_name_count; i++) {
		if (tdm_loglevel_names[i].id == level) {
			return tdm_loglevel_names[i].name;
		}
	}
	return NULL;
}

TDM_API const tdm_loglevel_t
tdm_loglevel_get_id(const char *name)
{
	for (int i = 0; i < tdm_loglevel_name_count; i++) {
		if (strcmp(tdm_loglevel_names[i].name, name) == 0) {
			return tdm_loglevel_names[i].id;
		}
	}
	return TDM_LOG_INVALID;
}

static const char* tdm_loglevel_get_color(const tdm_loglevel_t level)
{
	if (level < TDM_LOG_NONE || level >= TDM_LOG_INVALID) {
		return tdm_loglevel_colors[TDM_LOG_NONE];
	}
	return tdm_loglevel_colors[level];
}

/* */
static struct {
	tdm_loglevel_t level;
	int flags;
} tdm_log_global = {
	TDM_LOG_ERROR,
	TDM_LOGF_COLORS
};

/* public */
TDM_API tdm_status_t
tdm_set_loglevel(const tdm_loglevel_t level)
{
	if (level < TDM_LOG_NONE || level >= TDM_LOG_INVALID)
		return TDM_STATUS_INVALID;

	tdm_log_global.level = level;
	return TDM_STATUS_SUCCESS;
}

TDM_API tdm_loglevel_t
tdm_get_loglevel(void)
{
	return tdm_log_global.level;
}

/* public (maybe only modules api) */
TDM_API tdm_status_t
__tdm_log(const tdm_loglevel_t level, const void *p1, const void *p2, const char *fmt, ...)
{
	va_list ap;

	if (level > tdm_log_global.level)	/* ignore everything above current level */
		return TDM_STATUS_SUCCESS;

	va_start(ap, fmt);
	if (tdm_log_global.flags & TDM_LOGF_COLORS) printf("%s", tdm_loglevel_get_color(level));
	vprintf(fmt, ap);
	if (tdm_log_global.flags & TDM_LOGF_COLORS) printf(TDM_COLOR_NONE);
	va_end(ap);
	return TDM_STATUS_SUCCESS;
}

TDM_API tdm_status_t
__tdm_printf(const void *p1, const void *p2, const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	vprintf(fmt, ap);
	va_end(ap);
	return TDM_STATUS_SUCCESS;
}

tdm_status_t tdm_log_init(void)
{
/*
	memset(&tdm_log_global, 0, sizeof(tdm_log_global));
	tdm_log_global.level = TDM_LOG_ERROR;
	tdm_log_global.flags = TDM_LOGF_COLORS;
*/
	return TDM_STATUS_SUCCESS;
}
