/*
 *
 *
 */
#ifndef __TDM_MODULE_H__
#define __TDM_MODULE_H__

/**
 * Definition of module
 */
struct tdm_module_def {
	const char *name;
	const char *description;

	/* module ops */
	tdm_status_t (* module_load)(struct tdm_module *mod);		/*!< invoked after loading */
	tdm_status_t (* module_unload)(struct tdm_module *mod);		/*!< invoked before unloading */
	tdm_status_t (* module_probe)(struct tdm_module *mod);		/*!< device auto-detection */
	tdm_status_t (* module_configure)(struct tdm_module *mod);	/*!< module configuration / manual device registraion */
};

#ifdef LIBTDM_INTERNAL

/**
 * "Internal" module handle
 */
struct tdm_module {
	struct tdm_object obj;

	struct tdm_module_def *def;	/*!< module definition + ops */

	struct dlist_head     module_list;

	struct dlist_head     devices;
	unsigned int          nr_devices;

	char                  *path;	/*!< on-disk path */
};


tdm_status_t tdm_load_modules(void);

/* add module to global list, after successfull load + init */
tdm_status_t tdm_register_module(struct tdm_module *mod);
tdm_status_t tdm_unregister_module(struct tdm_module *mod);
#endif

tdm_status_t tdm_set_modules_dir(const char *path);

void tdm_list_modules(void);
struct tdm_module *tdm_find_module(const char *name);

const char *tdm_module_name(const struct tdm_module *mod);
const char *tdm_module_description(const struct tdm_module *mod);
const char *tdm_module_path(const struct tdm_module *mod);

#endif /* __TDM_MODULE_H__ */
