/*
 *
 *
 */
#ifndef __TDM_OBJECT_H__
#define __TDM_OBJECT_H__

typedef enum {
	TDM_OBJECT_NONE = 0,
	TDM_OBJECT_MODULE,
	TDM_OBJECT_DEVICE,
	TDM_OBJECT_PORT,
	TDM_OBJECT_PROTOCOL,
	TDM_OBJECT_SPAN,
	TDM_OBJECT_SPANGROUP,
	TDM_OBJECT_POLICY,
	TDM_OBJECT_CHANNEL,
	TDM_OBJECT_CALL,

	TDM_OBJECT_MAX	/* always last! */
} tdm_object_type_t;

/**
 * @note	Always first member of struct!
 */
struct tdm_object {
	char *name;
	struct tdm_object *owner;
	tdm_object_type_t type;

	unsigned long get_cnt, put_cnt;	/*!< reference counting */
};

#define tdm_object_is(x, t)	\
	(((struct tdm_object *)(x))->type == (t))

#define tdm_object_is_module(x)	\
	tdm_object_is(x, TDM_OBJECT_MODULE)

#define tdm_object_is_device(x)	\
	tdm_object_is(x, TDM_OBJECT_DEVICE)

#define tdm_object_is_port(x)	\
	tdm_object_is(x, TDM_OBJECT_PORT)

#define tdm_object_is_span(x)	\
	tdm_object_is(x, TDM_OBJECT_SPAN)

#define tdm_object_is_spangroup(x)	\
	tdm_object_is(x, TDM_OBJECT_SPANGROUP)

#define tdm_object_is_channel(x)	\
	tdm_object_is(x, TDM_OBJECT_CHANNEL)

#define tdm_object_is_protocol(x)	\
	tdm_object_is(x, TDM_OBJECT_PROTOCOL)

#define tdm_object_is_policy(x)	\
	tdm_object_is(x, TDM_OBJECT_POLICY)

#define tdm_object_name(x)	\
	(((struct tdm_object *)(x))->name)

#define tdm_object_type(x)	\
	(((struct tdm_object *)(x))->type)

#define tdm_object_owner(x)	\
	(((struct tdm_object *)(x))->owner)

#define tdm_object_set_type(x, t)	\
	(((struct tdm_object *)(x))->type = (t))

#define tdm_object_set_owner(x, o)	\
	((struct tdm_object *)(x))->owner = (struct tdm_object *)(o)

/* not sure if we want this */
#define tdm_object_use_count(x)	\
	(((struct tdm_object *)(x))->get_cnt - ((struct tdm_object *)(x))->put_cnt)

#define tdm_object_in_use(x)	\
	(tdm_object_use_count(x) != 0)

#define tdm_object_put(x)	\
	({ ((struct tdm_object *)(x))->put_cnt++; (x); })

#define tdm_object_get(x)	\
	({ ((struct tdm_object *)(x))->get_cnt++; (x); })


struct tdm_object *tdm_object_new(const char *name, const size_t size);
tdm_status_t tdm_object_rename(struct tdm_object *obj, const char *name);
void tdm_object_destroy(struct tdm_object *obj);


#endif /* __TDM_OBJECT_H__ */
