/*
 *
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <string.h>

#include "utils/dlist.h"

#include "tdm_common.h"
#include "tdm_object.h"
#include "tdm_log.h"

#include "tdm_protocol.h"


static DLIST_HEAD(tdm_protocol_list);


/**
 * signalling/mixed modules only: register new signalling type
 * @note	Usually only called from module_load
 */
tdm_status_t tdm_module_register_protocol(struct tdm_module *mod, const struct tdm_protocol_def *def)
{
	struct tdm_protocol *proto = NULL;

	if (!mod || !tdm_object_is_module(mod) || !def)
		return TDM_STATUS_GENERR;

	proto = (struct tdm_protocol *)tdm_object_new(def->name, sizeof(*proto));
	if (!proto)
		return TDM_STATUS_GENERR;

	tdm_object_set_type(proto, TDM_OBJECT_PROTOCOL);
	tdm_object_set_owner(proto, mod);

	proto->def = (struct tdm_protocol_def *)def;

	/* TODO: lock tdm_protocol_list */

/*	if (tdm_hash_insert(name, proto) != TDM_STATUS_SUCCESS) {
		tdm_error("%s: failed to register protocol\n", name);
		goto error;
	}
*/
	dlist_insert_tail(&tdm_protocol_list, &proto->protocol_list);

	/* TODO: unlock tdm_protocol_list */
	return TDM_STATUS_SUCCESS;

error:
	tdm_object_destroy((struct tdm_object *)proto);
	return TDM_STATUS_GENERR;
}

/**
 * @note	usually called only from module_unload callback
 */
tdm_status_t tdm_protocol_unregister(struct tdm_protocol *proto)
{
	if (!proto || !tdm_object_is_protocol(proto))
		return TDM_STATUS_GENERR;

	/* TODO: check if protocol is still in use */
	if (tdm_object_in_use(proto)) {
		tdm_error("%s: protocol still in use\n", tdm_object_name(proto));
		return TDM_STATUS_GENERR;
	}

	/* TODO: lock protocol_list */

//	tdm_hash_remove(tdm_object_name(proto));

	dlist_remove(&proto->protocol_list);

	/* TODO: unlock protocol_list */

	return TDM_STATUS_SUCCESS;
}

struct tdm_protocol *tdm_protocol_find(const char *name)
{
	struct dlist_head *entry;

	dlist_for_each(&tdm_protocol_list, entry) {
		struct tdm_protocol *proto = dlist_entry(entry, struct tdm_protocol, protocol_list);

		if (!tdm_object_is_protocol(proto)) {
			tdm_error("%p: object in protocol list is not a protocol", proto);
			break;
		}

		if (!strcmp(tdm_object_name(proto), name))
			return proto;
	}
	return NULL;
}
