/*
 *
 *
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>

#include "utils/dlist.h"

#include "tdm_common.h"
#include "tdm_object.h"
#include "tdm_log.h"

#include "tdm_spangroup.h"
#include "tdm_span.h"
#include "tdm_policy.h"


void tdm_spangroup_set_private(struct tdm_spangroup *group, void *data)
{
	if (!group || !tdm_object_is_spangroup(group))
		return;

	group->priv = data;
}

void *tdm_spangroup_get_private(struct tdm_spangroup *group)
{
	if (!group || !tdm_object_is_spangroup(group))
		return NULL;

	return group->priv;
}

struct tdm_spangroup *tdm_spangroup_new(const char *name)
{
	struct tdm_spangroup *group = NULL;

	group = (struct tdm_spangroup *)tdm_object_new(name, sizeof(*group));
	if (!group)
		return NULL;

	tdm_object_set_type(group, TDM_OBJECT_SPANGROUP);

	dlist_init_head(&group->spans);

	return group;
}

tdm_status_t tdm_spangroup_add_span(struct tdm_spangroup *group, struct tdm_span *span)
{
	tdm_status_t retval = TDM_STATUS_SUCCESS;

	if (!group || !tdm_object_is_spangroup(group))
		return TDM_STATUS_GENERR;

	if (!span || !tdm_object_is_span(span))
		return TDM_STATUS_GENERR;

	/* TODO: check if span is already a member of a/this group */

	/* TODO: locking */

	dlist_insert_tail(&group->spans, &span->group_span_list);
	group->nr_spans++;
	span->group = group;

	/* TODO: locking */

	/* */
	if (group->policy->def->group_add_span) {
		if ((retval = group->policy->def->group_add_span(group, span)) != TDM_STATUS_SUCCESS) {
			tdm_error("%s: failed to add span \"%s\"\n", tdm_object_name(group), tdm_object_name(span));
			goto error;
		}
	} else {
		tdm_debug("%s: no add span callback\n", tdm_object_name(group));
	}

	return TDM_STATUS_SUCCESS;

error:
	dlist_remove(&span->group_span_list);
	group->nr_spans--;
	span->group = NULL;
	return retval;
}

int tdm_spangroup_has_span(struct tdm_spangroup *group, struct tdm_span *span)
{
	if (!group || !tdm_object_is_spangroup(group))
		return TDM_FALSE;

	if (!span || !tdm_object_is_span(span))
		return TDM_FALSE;

	return (dlist_get_head(&span->group_span_list) == dlist_get_head(&group->spans));
}

tdm_status_t tdm_spangroup_remove_span(struct tdm_spangroup *group, struct tdm_span *span)
{
	if (!group || !tdm_object_is_spangroup(group))
		return TDM_STATUS_GENERR;

	if (!span || !tdm_object_is_span(span))
		return TDM_STATUS_GENERR;

	if (!tdm_spangroup_has_span(group, span)) {
		tdm_error("%s: not a member of spangroup \"%s\"\n", tdm_object_name(span), tdm_object_name(group));
		return TDM_STATUS_GENERR;
	}

	/* */
	if (group->policy->def->group_remove_span) {
		tdm_status_t retval;
		if ((retval = group->policy->def->group_remove_span(group, span)) != TDM_STATUS_SUCCESS) {
			tdm_error("%s: failed to remove span \"%s\"\n", tdm_object_name(group), tdm_object_name(span));
			return retval;
		}
	} else {
		tdm_debug("%s: no remove span callback\n", tdm_object_name(group));
	}

	/* TODO: locking */

	dlist_remove(&span->group_span_list);
	group->nr_spans--;
	span->group = NULL;

	/* TODO: locking */

	return TDM_STATUS_SUCCESS;
}
